/**
 * @function _scaledSize
 */

"use strict";

import numcal from 'numcal';

function _scaledSize(contentSize, frameSize, policy) {
    let cw = contentSize.width,
        ch = contentSize.height,
        fw = frameSize.width,
        fh = frameSize.height;

    let wRate = fw / cw,
        hRate = fh / ch;

    switch (policy) {
        case 'none':
        {
            return contentSize;
        }
        case 'fit':
        {
            let rate = numcal.min(wRate, hRate);
            return {
                width: contentSize.width * rate,
                height: contentSize.height * rate
            }
        }
        case 'fill':
        {
            let rate = numcal.max(wRate, hRate);
            return {
                width: contentSize.width * rate,
                height: contentSize.height * rate
            }
        }
        default:
        {
            throw new Error(`Unknown policy: ${policy}`);
        }
    }
}

module.exports = _scaledSize;
