/**
 * apeman react package for image component.
 * @constructor ApImage
 */

"use strict";


import React, {PropTypes as types} from 'react';
import ReactDOM from 'react-dom';
import classnames from 'classnames';
import numcal from 'numcal';
import _scaledSize from './_scaled_size';
import {ApSpinner} from 'apeman-react-spinner';
import {ApPureMixin} from 'apeman-react-mixins';

/** @lends ApImage */
let ApImage = React.createClass({


    //--------------------
    // Specs
    //--------------------

    propTypes: {
        scale: types.oneOf([
            'fit',
            'fill',
            'none'
        ]),
        width: types.oneOfType([types.number, types.string]),
        height: types.oneOfType([types.number, types.string]),
        src: types.string,
        alt: types.string,
        spinnerTheme: types.string,
        onLoad: types.func,
        onError: types.func
    },

    mixins: [
        ApPureMixin
    ],

    statics: {
        scaledSize: _scaledSize,
        zeroIfNaN(value) {
            return isNaN(value) ? 0 : value;
        },
        nullIfNaN(value) {
            return isNaN(value) ? null : value;
        }
    },

    getInitialState() {
        let s = this;
        return {
            imgWidth: null,
            imgHeight: null,
            mounted: false,
            ready: false,
            loading: !!s.props.src,
            error: null
        };
    },

    getDefaultProps() {
        return {
            scale: 'none',
            width: null,
            height: null,
            src: null,
            alt: "NO IMAGE",
            spinnerTheme: ApSpinner.DEFAULT_THEME,
            onLoad: null,
            onError: null
        };
    },

    render() {
        let s = this,
            {state, props} = s;

        let size = {
            width: props.width || null,
            height: props.height || null
        };

        let {mounted, error, ready, loading} = state;
        return (
            <div className={ classnames('ap-image', props.className, {
                'ap-image-loading': props.src && loading,
                'ap-image-ready': props.src && ready
            }) }
                 style={ Object.assign({}, size, props.style) }>
                { mounted && error ? s._renderNotfound(size) : null}
                { mounted && !error ? s._renderImg(size, mounted) : null }
                { loading ? s._renderSpinner(size) : null }
            </div>
        );
    },


    //--------------------
    // Lifecycle
    //--------------------

    componentWillMount() {
        let s = this;
    },

    componentDidMount() {
        let s = this;
        s.setState({
            mounted: true
        });

        setTimeout(() => {
            s.resizeImage();
        }, 0);
    },

    componentWillReceiveProps(nextProps) {
        let s = this;

        let src = s.props.src,
            nextSrc = nextProps.src;
        let srcChanged = !!nextSrc && (nextSrc !== src);
        if (srcChanged) {
            s.setState({
                ready: false,
                loading: true,
                error: null
            });
        }

    },

    componentWillUpdate(nextProps, nextState) {
        let s = this;
        s.resizeImage();
    },

    componentDidUpdate(prevProps, prevState) {
        let s = this;
    },

    componentWillUnmount() {
        let s = this;
    },

    //------------------
    // Helper
    //------------------

    handleLoad(e) {
        let s = this,
            {props} = s;

        if (props.onLoad) {
            props.onLoad(e);
        }

        s.resizeImage(e.target.width, e.target.height);
    },

    handleError(e) {
        let s = this,
            {props} = s;

        s.setState({
            error: e,
            loading: false
        });

        if (props.onError) {
            props.onError(e);
        }
    },

    resizeImage(imgContentWidth, imgContentHeight){
        let s = this,
            {state, props} = s;

        imgContentWidth = imgContentWidth || state.imgContentWidth;
        imgContentHeight = imgContentHeight || state.imgContentHeight;

        let valid = imgContentWidth && imgContentHeight;
        if (!valid) {
            return;
        }

        let elm = ReactDOM.findDOMNode(s);
        let frameSize = {
                width: elm.offsetWidth,
                height: elm.offsetHeight
            },
            contentSize = {
                height: imgContentHeight,
                width: imgContentWidth
            },
            scaledSize = ApImage.scaledSize(
                contentSize, frameSize, props.scale
            );

        s.setState({
            imgContentWidth: imgContentWidth,
            imgContentHeight: imgContentHeight,
            imgWidth: scaledSize.width,
            imgHeight: scaledSize.height,
            ready: true,
            loading: false
        });
    },

    //------------------
    // Private
    //------------------
    _renderImg(size) {
        let s = this,
            {state, props} = s;

        let {nullIfNaN, zeroIfNaN} = ApImage;

        return (
            <img src={ props.src }
                 alt={ props.alt }
                 className={ classnames('ap-image-content') }
                 style={ {
                        top:zeroIfNaN((size.height - state.imgHeight) / 2),
                        left:zeroIfNaN((size.width - state.imgWidth) / 2),
                        width:nullIfNaN(state.imgWidth),
                        height:nullIfNaN(state.imgHeight)
                     } }
                 onLoad={ s.handleLoad }
                 onError={ s.handleError }
            />
        );
    },
    _renderNotfound(size) {
        let s = this,
            {props} = s;

        return (
            <div className="ap-image-notfound"
                 style={ {
                    lineHeight:`${size.height}px`,
                    fontSize:`${numcal.min(size.height * 0.5, 24)}`
                 } }
            >{ props.alt }</div>
        );
    },
    _renderSpinner(size) {
        let s = this,
            {props} = s;

        return (
            <ApSpinner className="ap-image-spinner"
                       theme={ props.spinnerTheme }
                       style={ {
                       width: size.width,
                       height:size.height
                       } }/>
        );
    }
});

module.exports = ApImage;